from .datastorage import DataStorage


class Hdfs(DataStorage):

    # overriding abstract method
    def load_dataset(self, spark, read_all=True, input_dest='', header=True):

        if input_dest == '':
            input_dataset = self.args["input-dataset"]
        else:
            input_dataset = input_dest
        hdfs_host = self.args["hdfsHost-input-dataset"]
        hdfs_port = self.args["hdfsPort-input-dataset"]
        delimiter = self.args["delimiter"] if "delimiter" in self.args else ","
        input_file_format = self.args["inputFileFormat"] if "inputFileFormat" in self.args else "csv"

        return spark.read.load("hdfs://"+hdfs_host+":"+hdfs_port+input_dataset, format=input_file_format, sep=delimiter, inferSchema=True, header=header, error_bad_lines=False)

    # overriding abstract method
    def save_dataset(self, df, output_dest):

        if output_dest == '':
            output_dataset = self.args["output-dataset"]
        else:
            output_dataset = output_dest
        delimiter = self.args["delimiter"] if "delimiter" in self.args else ","
        output_file_format = self.args["outputFileFormat"] if "outputFileFormat" in self.args else "csv"

        df.write.options(delimiter=delimiter).\
            mode('overwrite').\
            option("header", True).\
            save("hdfs://"+self.args["hdfsHost-output-dataset"]+":"+self.args["hdfsPort-output-dataset"]+output_dataset, format=output_file_format)



from .datastorage import DataStorage

import os


class Hive(DataStorage):

    # overriding abstract method
    def load_dataset(self, spark, read_all=True, input_dest='', header=True):

        input_columns = self.args["input-columns"] if "input-columns" in self.args else "*"

        if input_dest == '':
            input_dataset = self.args["input-dataset"]
        else:
            input_dataset = input_dest

        # To manage supervised algorithms
        if "labelCol" in self.args:
            label_col = self.args["labelCol"]
            columns_list = input_columns.split(",")
            if label_col not in columns_list:
                columns_list.append(label_col)
            input_columns = ','.join(columns_list)

        if read_all:
            df = spark.sql("SELECT * FROM " + input_dataset)
        else:
            df = spark.sql("SELECT " + input_columns + " FROM " + input_dataset)

        # rename column
        for column in input_columns.split(","):
            df = df.withColumnRenamed(column.lower(), column)
        return df

    # overriding abstract method
    def save_dataset(self, df, output_dest):

        if output_dest == '':
            output_dataset = self.args["output-dataset"]
        else:
            output_dataset = output_dest

        os.environ["HADOOP_USER_NAME"] = self.args["hiveHadoopUserName-output-dataset"]

        df.write.mode("overwrite").saveAsTable(output_dataset)



import setuptools

REQUIREMENTS = [i.strip() for i in open("requirements.txt").readlines()]

setuptools.setup(
    name='pyspark-utilities',
    version='0.0.7',
    description='Spark utilities to be used by the Analytics BDA',
    author='ENG ALIDA TEAM',
    author_email='engineering-alida-lab@eng.it',
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=REQUIREMENTS
)



import importlib
import sys


def get_args():

    args = {}
    for _ in sys.argv[1:]:
        if "=" in _:
            p = _.split('=')
            args[p[0].replace("--", "", 1)] = p[1]

    if args["dataStorageType-input-dataset"].lower().startswith("presto"):
        args["input-columns"] = args["input-columns"] if "input-columns" in args else "*"

    return args


def get_class(module_name):

    module = importlib.import_module("." + module_name, "pysparkutilities")
    return getattr(module, module_name.capitalize())


def load_dataset(sc, read_all, input_dest='', header=True):
    # sc -> Spark context
    # read_all -> True to read all data, False to read 'input-columns' in 'args'

    args = get_args()
    data_storage = args["dataStorageType-input-dataset"]
    module_name = data_storage.split('-')[0].lower()

    class_ = get_class(module_name)
    instance = class_(args)

    return instance.load_dataset(sc, read_all, input_dest, header)


def save_dataset(df, output_dest=''):
    # df -> Spark dataframe to store
    # output_dest -> destinaion output, if not specified it takes 'output-dataset' destinaion from 'args'

    args = get_args()
    data_storage = args["dataStorageType-output-dataset"]
    module_name = data_storage.split('-')[0].lower()

    class_ = get_class(module_name)
    instance = class_(args)

    return instance.save_dataset(df, output_dest)


def lower_columns_list(columns_list):

    return [x.lower() for x in columns_list]
